# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

################################################################################
############ Moduli da importare ###############################################
################################################################################
import os
import datetime
import csv
import gtk
import gobject
import cairo
import pango
from preventares.preventaresconfig import get_data_path
from preventares.preventa_lib import DEFAULT_SETTINGS as PREVENTA_SETTINGS
################################################################################
############ SUPPORTO INTERNAZIONALIZZAZIONE  ##################################
################################################################################
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')
################################################################################
############ IMPOSTAZIONI LOGGINGS  ############################################
################################################################################
import logging
LOGGING = "PreventaLib"
logging.getLogger(LOGGING)
################################################################################
############ SUPPORTO PER LA LOCALIZZAZIONE  ###################################
################################################################################
import locale
locale.setlocale(locale.LC_ALL, '')
################################################################################
############ IMPOSTAZIONE COSTANTI  ############################################
################################################################################
# immagine di default del campo intestazione
DEFAULT_IMAGE = os.path.join(get_data_path(), "media", "preventares.png")
# dimensione massima degli offset della pagina
MAX_OFFSET = 20
MAX_HEADING = 0.2
################################################################################
############ IMPOSTAZIONE SETTAGGI DI DEFAULT  #################################
################################################################################
# impostazioni di default
DEFAULT_SETTINGS = {
                    # impostazioni generali di stampa 
                    "print_first_page":True,
                    "print_logo":True,
                    "print_date":True,
                    "new_chapter_change_page":True,
                    "print_only_if_quantity_not_0":False,
                    "print_without_chapter": False,
                    "print_only_grand_total": False,
                    "print_report_by_chapter": False,
                    "print_analisi": False,
                    "add_to_pages_number": 0,
                    # modalita di stampa delle descrizioni degli articoli
                    "computo_print_descr_voce":True,
                    "computo_print_descr_estesa":True,
                    "computo_print_note":True,
                    "computo_print_price":True,
                    # impostazione dei font di stampa:
                    "font_type":"sans 10",
                    "font_type_title":"sans 25",
                    "line_spacing": 0,                    
                    # impostazioni di formattazione pagina e numeri:
                    "owner_image":DEFAULT_IMAGE,
                    "grid_color": "#A52121",
                    "grid_line_width": 1.0,
                    "page_offset_top_x": 10,
                    "page_offset_top_y": 10,
                    "page_offset_bottom_x": 10,
                    "page_offset_bottom_y": 10,
                    "decimals": PREVENTA_SETTINGS["decimals"],
                    # campi validi per le descrizioni sono:
                    # 'i' = italic, 'b'= bold, 'u'= underline, 'big'= grande, 'small'= piccolo
                    "description_code_markup": "b",
                    "description_extended_markup": "",
                    "description_voice_markup": "i",
                    "description_note_markup": "i",
                    "description_analysis_markup": "i",
                    "main_title":"",
                    "blacklist_sup_cat":"",
                    "blacklist_mid_cat":"",
                    "blacklist_sub_cat":"",
                    "blacklist_sup_cap":"",
                    "blacklist_mid_cap":"",
                    "blacklist_sub_cap":""}

DEFAULT_TITLE =    {"computo":_("Computo Metrico"),
                    "epu":_("Elenco Prezzi Unitari"),
                    "lavorazioni":_("Lista delle Lavorazioni"),
                    "analisi": _("Analisi Prezzi"), 
                    "report-cmp": _("Riepilogo Categorie di Computo"), 
                    "report-epu": _("Riepilogo Capitoli di Elenco Prezzi"),
                    "report-inc_mo": _("Riepilogo Incidenza della Manodopera"),
                    "report-inc_sic": _("Riepilogo Incidenza della Sicurezza")}
################################################################################
############ classe per la stampa PrintPreventa ################################
################################################################################

class PrintPreventa:
    def __init__(self, preventivo, tipo_stampa, print_settings=None, 
                 categorie=None, capitoli=None):
        self.preventivo = preventivo
        # valori accettati della CLASSE: "computo","epu", "lavorazioni", "analisi"
        self.tipo_stampa = tipo_stampa
        # capitoli e categorie da stampare
        self.categorie = categorie
        self.capitoli = capitoli
        # impostazione dei settaggi di stampa
        global settings
        settings = DEFAULT_SETTINGS
        self.__set_printing_settings(print_settings, settings)
        # impostazione del titolo
        self.title = settings.get("main_title", "")
        if self.title == "":
            self.title = DEFAULT_TITLE.get(tipo_stampa, "")
        # inizio delle procedure di stampa
        print_op = gtk.PrintOperation()
        # imposta le operazioni di stampa con quelle salvate
        print_op.set_print_settings(print_settings)
        # imposta la pagina per la stampa
        page_setup = gtk.PageSetup()
        # ricava le impostazioni della gtk.PageSetup da gtk.PrintSettings
        if print_settings is not None:
            if print_settings.get_orientation() is not None:
                page_setup.set_orientation(print_settings.get_orientation())
            if print_settings.get_paper_size() is not None:
                page_setup.set_paper_size_and_default_margins(
                                print_settings.get_paper_size())
        # trasferisci le impostazioni di pagina a gtk.PrintOperation
        print_op.set_default_page_setup(page_setup)
        # attiva la barra di avanzamento durante le operazioni di stampa
        print_op.set_show_progress(True)
        # imposta le funzioni richiamate (callbacks) per impaginazione e stampa
        print_op.connect("begin-print", self.begin_print)
        print_op.connect("paginate", self.paginate)
        print_op.connect("draw_page", self.draw_page)
        print_op.connect("end_print", self.end_print)
        print_op.connect("done", self.done_print)
        # imposta il nome del lavoro di stampa
        print_op.set_job_name("%s-%s" % (preventivo.get_filename(), datetime.datetime.now()))
        # imposta l'unità di misura del contesto di stampa
        print_op.set_unit(gtk.UNIT_PIXEL) #TODO verificare se ok x windows, altrimenti usare gtk.UNIT_POINTS
        # avvia le operazioni di stampa
        response = print_op.run(gtk.PRINT_OPERATION_ACTION_PRINT_DIALOG, None)
        # Gestione degli errori di stampa con finestra di dialogo
        if response == gtk.PRINT_OPERATION_RESULT_ERROR:
           error_dialog = gtk.MessageDialog(parent, gtk.DIALOG_DESTROY_WITH_PARENT, 
                 gtk.MESSAGE_ERROR, gtk.BUTTONS_CLOSE, _("Error printing file:\n"))
           error_dialog.connect("response", lambda w,id: w.destroy())
           error_dialog.show()
        elif response == gtk.PRINT_OPERATION_RESULT_APPLY:
           # salva le operazioni di stampa
           print_op_settings = print_op.get_print_settings()
           # esegui il merge tra le opzioni della stampante e selezione della opzioni
           print_op_settings.foreach(self.__save_settings, self.print_settings)
        elif response == gtk.PRINT_OPERATION_RESULT_CANCEL:
           pass

################################################################################
########### IMPOSTAZIONE DI STAMPA #############################################
################################################################################

    def __set_printing_settings(self, print_settings, settings):
        """Imposta i valori del dizionario 'settings' contenente i parametri di stampa"""
        # se le impostazioni sono nulle usa le impostazioni di default
        if print_settings is None:
            settings = DEFAULT_SETTINGS
            self.print_settings = gtk.PrintSettings()
        # se le impostazioni sono delle gtk.PrintSettings sostituisci quelle di default
        elif type(print_settings) is type(gtk.PrintSettings()):
            self.print_settings = print_settings
            self.print_settings.foreach(self.__setup_key, user_data=settings)
        # numero di pagina da cui partire
        self.pages_numer_to_add = int(settings["add_to_pages_number"])
        # impostazioni variabili relative ai font
        self.font = settings["font_type"]
        self.font_title = settings["font_type_title"]
        font_lst = self.font.split(" ")
        self.font_type = font_lst[0]
        self.font_size = int(font_lst[-1])
        self.owner_image = settings["owner_image"]
        self.line_spacing = settings["line_spacing"]
        # impostazione relativa ai colori di default della griglia
        color = gtk.gdk.color_parse(settings["grid_color"])
        self.color_rgb = (color.red_float, color.green_float, color.blue_float)
        # spessore delle linee di default
        self.width = float(settings["grid_line_width"])
        return settings

    def __setup_key(self, key, value, settings):
        if value == 'false':
            value = False
        elif value == 'true':
            value = True
        settings[key] = value
        return key, value

    def __save_settings(self, key, value, print_settings):
        print_settings.set(key, value)
        return key, value

################################################################################
########### CHIAMATE PER LA GESTIONE OPERAZIONI DI STAMPA ######################
################################################################################

    def begin_print(self, operation, context, user_param=None):
        """Inizio delle operazioni di stampa e calcolo delle pagine necessarie per la stampa"""
        # impostazioni dimensioni della pagina e intestazione
        global BORDER_x
        global BORDER_y
        global HEADING_TOP
        global HEADING_BOTTOM
        X0, Y0 = self.__get_bottom_offset()
        BORDER_x = context.get_width() - X0
        BORDER_y = context.get_height() - Y0
        HEADING_TOP = self.__get_top_heading(context)
        HEADING_BOTTOM = BORDER_y
        # ricavare il numero di PIXEL per pagina (è una costante)
        self.n_pixel_pagina = HEADING_BOTTOM
        # ricavare la larghezza max delle descrizioni a seconda delle impostazioni di pagina
        self.max_width = 0.52 * BORDER_x * pango.SCALE
        # messaggi di debug per la dimensione della pagina
        logging.debug("Page Size: X = {0}, Y = {1}".format(BORDER_x, BORDER_y)) 
        logging.debug("Page max pixel Nr.:'{0}' PIXEL".format(self.n_pixel_pagina))
        logging.debug("Text max_width: '{0}' pango.UNIT".format(self.max_width))
        # inizializza il dizionario per memorizzare i layout delle pagine impaginate
        self.__layout_page = dict()
        # lista per memorizzare gli articoli incompleti da stampare nella nuova pagina durante i salti pagina
        self.__layout_articoli_riportati = list()
        # ricavare la stampa totale del computo
        self.num_pagine = 0
        self.lista_articoli_stampare = list()
        self.articolo_stampare = 0
        self.lista_report = list()
        self.rigo_report_stampare = 0
        # ricavare la lista di articoli da stampare dal preventivo
        self.lista_articoli_stampare, self.lista_report = get_output_to_print(
                                    self.preventivo, settings, self.tipo_stampa,
                                    categorie=self.categorie, capitoli=self.capitoli)
        # se richiesta la prima pagina aumenta di 1 il numero di pagine
        if settings["print_first_page"]:
            self.num_pagine += 1
            operation.set_n_pages(self.num_pagine)
        return

    def paginate(self, operation, context, user_param=None):
        # ricavare il numero di righe totale e il layout_descrizione
        if self.tipo_stampa in ["computo", "lavorazioni", "epu", "analisi"]:
            if self.paginate_computo(operation, context, user_param=None) == True:
                return self.paginate_report(operation, context, user_param=None)
            else: return False
        elif self.tipo_stampa in ["report-cmp", "report-epu"]:
            return self.paginate_report(operation, context, user_param=None)
        elif self.tipo_stampa in ["report-inc_mo", "report-inc_sic"]:
            return self.paginate_report_incidenze(operation, context, user_param=None)
        else:
            return True

    def draw_page(self, operation, context, page_nr, user_param=None):
        """Callback per la gestione delle pagine da stampare"""
        # carica in memoria il numero totale di pagine da stampare
        self.n_pages_to_print = operation.get_n_pages_to_print()
        # stampa la prima pagina se le impostazioni lo prevedono
        if page_nr == 0 and settings["print_first_page"]:
            self.first_page(context, page_nr, user_param=self.title)
            return
        else:
            # inserisco intestazione inizio pagina
            self.heading_page_top(context, self.title, user_param=None)
            # inserisco intestazione fine pagina
            self.heading_page_bottom(context, page_nr, user_param=None)
            # imposto il layout di pagina da stampare
            lista_layout_pagina = self.__layout_page[page_nr+1] # la pagina 0 è riservata alla pag.presentazione
            if self.tipo_stampa in ["computo", "epu", "lavorazioni"]:
                # se il layout ha 4 campi significa che è un report
                if len(lista_layout_pagina[0]) == 4:
                    self.draw_report_page(context, lista_layout_pagina)
                else:
                    self.draw_computo_page(context, lista_layout_pagina)
            elif self.tipo_stampa == "analisi":
                self.draw_analisi_page(context, lista_layout_pagina)
            elif self.tipo_stampa in ["report-cmp", "report-epu"]:
                self.draw_report_page(context, lista_layout_pagina)
            elif self.tipo_stampa in ["report-inc_mo", "report-inc_sic"]:
                self.draw_incidenze_page(context, lista_layout_pagina, self.tipo_stampa)
            return

    def end_print(self, operation, context, user_param=None):
        """Fine delle operazioni di stampa"""
        # informazioni di debug
        logging.debug("Total Page Nr.:'{0}'".format(self.num_pagine))
        logging.debug("Total Art. printed Nr.:'{0}'".format(len(self.lista_articoli_stampare)))
        # azzera tutte le variabili e le liste in memoria
        del self.__layout_page
        del self.__layout_articoli_riportati
        del self.lista_articoli_stampare
        del self.lista_report
        del self.rigo_report_stampare
        del self.articolo_stampare
        del self.num_pagine
        #azzera il 'context' in memoria
        del context
        return True

    def done_print(self, operation, result, user_param=None):
        """Istruzioni eseguite quando le Operazioni di stampa sono concluse"""
        logging.debug(result)
################################################################################
########### INTESTAZIONE DELLE PAGINE ##########################################
################################################################################

    def heading_page_top(self, context, testo=None, user_param=None):
        cairo_context = context.get_cairo_context()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        widht = BORDER_x * pango.SCALE
        # inserisce il logo a inizio pagina
        if settings["print_logo"]:
            height = self.draw_pixmap(context, self.owner_image, x0, y0)
        else: height = 0
        # inserisce il titolo del documento
        if testo is not None:
            testo = "<b>{0}</b>".format(testo)
            if not settings["print_logo"]:
                y = y0
            else: 
                y = y0+height
            self.print_text(context, cairo_context, x0, y, 
                            testo, alignment=0, width=widht)
        # inserisce la data del documento
        if settings["print_date"]:
            date = self.__get_current_date()
            self.print_text(context, cairo_context, 0, y0, date,
                            alignment=1, width=widht)
        return

    def heading_page_bottom(self, context, page_nr, user_param=None):
        cairo_context = context.get_cairo_context()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        widht = BORDER_x * pango.SCALE
        # impostazione del testo per la stampa
        text = "{0:d}/{1:d}".format(page_nr+1+self.pages_numer_to_add, 
                          self.n_pages_to_print+self.pages_numer_to_add)
        layout = self.print_text_to_layout(context, text, alignment=0, width=widht)
        cairo_context.move_to(x0,BORDER_y)
        cairo_context.show_layout(layout)
        return

################################################################################
########### PRIMA PAGINA #######################################################
################################################################################
    def first_page(self, context, page_nr, user_param=None):
        """Routine per disegnare la copertina del computo"""
        cairo_context = context.get_cairo_context()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        x1 = BORDER_x
        y1 = BORDER_y
        # imposta il titolo da stampare
        if user_param is None:
            user_param = self.title
        title = """<span weight="ultrabold" size="xx-large">%s</span>""" % user_param
        # inserisce il logo a inizio pagina
        if settings["print_logo"]:
            h = self.draw_pixmap(context, self.owner_image, x0, y0)
            self.draw_line(context, x0, y0+h, x1, y0+h)
        # Estrae dal database i dati generali
        dati_generali = self.preventivo.dati_generali_list()
        if settings["print_date"]: 
            data = self.__get_current_date()
            data_desc = _("<i>Data:</i>")
        else: 
            data = str()
            data_desc = str()
        # imposta i font per i layout
        font_desc_size = 12
        font_desc = self.font_title + " " + str(font_desc_size)
        # stampa il titolo
        layout0 = self.print_text_to_layout(context, title, font=self.font_title, 
                                width=(BORDER_x * pango.SCALE), indent=None, alignment=0)
        width0, height0 = layout0.get_pixel_size()
        y = y0+(0.192*y1) #150
        x = x0
        cairo_context.move_to(x, y)
        cairo_context.show_layout(layout0)
        # stampa descrizione computo
        x = x0+(0.0356*x1) #20
        y += height0+(0.255*y1) #200
        testi_descr = [(_("<b>Oggetto:</b>"), dati_generali[4]), (_("<b>Indirizzo:</b>"),dati_generali[5]),
                       (_("<b>Comune:</b>"), dati_generali[6]), (_("<b>Provincia:</b>"),dati_generali[7]),
                       (_("<b>Committente:</b>"), dati_generali[8]), (_("<b>Redattore:</b>"),dati_generali[9]), 
                       (data_desc, data)]
        for t in testi_descr:
            layout1 = self.print_text_to_layout(context, t[0], font=font_desc, width=1.5*self.max_width)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout1)
            layout2 = self.print_text_to_layout(context, t[1], font=font_desc, width=1.5*self.max_width)
            width, height = layout2.get_pixel_size()
            cairo_context.move_to(x+100, y)
            cairo_context.show_layout(layout2)
            y += height+1.5*font_desc_size
        # crea le linee del bordo orizzontali
        self.draw_line(context, x0, y0, x1, y0)
        self.draw_line(context, x0, y1, x1, y1)
        # crea le linee del bordo verticali
        self.draw_line(context, x0, y0, x0, y1)
        self.draw_line(context, x1, y0, x1, y1)
        # inserisco intestazione fine pagina
        self.heading_page_bottom(context, page_nr, user_param=None)
        return

################################################################################
########### IMPAGINAZIONE E STAMPA PER COMPUTO METRICO - EPU - LAVORAZIONI #####
################################################################################

    def paginate_computo(self, operation, context, user_param=None):
        """
        Procedura di impaginazione per il COMPUTO-EPU-LAVORAZIONI e imposta il numero di pagine
        viene richiamata ad ogni cambio pagina finchè non ritorna 'False'
        """
        # inizializza la lista degli articoli di pagina
        lista_layout_articoli = list()
        # riporta gli articoli incompleti da stampare
        pixel_ya = 0
        if len(self.__layout_articoli_riportati) > 0:
            # calcola i pixel degli articoli riportati
            pixel_xa, pixel_ya = self.__layout_articoli_riportati[1].get_pixel_size()
            lista_layout_articoli.append(self.__layout_articoli_riportati)
        # azzera gli articoli riportati nella pagina
        self.__layout_articoli_riportati = tuple()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # inizializzo il numero di pixel stampati della pagina e spazio per intestazione
        n_pixel_stampati = HEADING_TOP + pixel_ya
        # inizializzo la lista degli articoli da stampare
        lista = self.lista_articoli_stampare [self.articolo_stampare:]
        # imposta la larghezza da usare per impaginare la descrizione
        if self.tipo_stampa == "analisi":
            width = self.max_width*0.65
        else: width = self.max_width
        # stampa del computo:
        for articolo in lista:
            # campi della lista da stampare #code, tariffa, Descrizione, um, quantita, pr_un, pr_tot, costo, ric, tempo, sic
            tipo = articolo[0]
            tariffa = str(articolo[1])
            um = articolo[3]
            quantita =  self.__convert_number_to_string(articolo[4], round_to_integer = True, decimals = 3)
            prezzo_unitario = self.__convert_number_to_string(articolo[5])
            prezzo_totale = self.__convert_number_to_string(articolo[6])
            costo_mat =  self.__convert_number_to_string(articolo[7])
            ricarico = self.__convert_number_to_string(articolo[8], decimals = 3)
            tempo_inst = self.__convert_number_to_string(articolo[9], round_to_integer = True, decimals = 3)
            sicurezza = self.__convert_number_to_string(articolo[10])
            descr = articolo[2]
            descr_codice, descr_estesa, descr_voce, note = descr[0], descr[1], descr[2], descr[3]
            # stampa codice e tariffa
            layout_tariffa = self.print_text_to_layout(context, tariffa, width=0.15*self.max_width)
            # stampa um
            layout_um = self.print_text_to_layout(context, str(um), width=0.15*self.max_width)
            # stampa quantita
            layout_quantita = self.print_text_to_layout(context, quantita)
            # stampa prezzo unitario
            layout_prezzo_unitario = self.print_text_to_layout(context, prezzo_unitario)
            # stampa prezzo totale
            layout_prezzo_totale = self.print_text_to_layout(context, prezzo_totale)
            # stampa costo materiali
            layout_costo_mat = self.print_text_to_layout(context, costo_mat)
            # stampa ricarico
            layout_ricarico = self.print_text_to_layout(context, ricarico)
            # stampa tempo installazione
            layout_tempo_inst = self.print_text_to_layout(context, tempo_inst)
            # stampa sicurezza
            layout_sicurezza = self.print_text_to_layout(context, sicurezza)
            # crea il testo descrizione dell'articolo con markup
            if articolo[0].startswith("$$$LIS$$$"): voce_analisi = True
            else: voce_analisi = False
            text_desc = self.__format_description(descr_codice, descr_estesa, descr_voce, note, voce_analisi=voce_analisi)
            #inserisci l'indentazione per le voci di listino
            if voce_analisi: ind=5000
            else: ind=None
            # crea il testo per la descrizione dell'articolo
            layout_descrizione = self.print_text_to_layout(context, text_desc, width=width, indent=ind)
            # verifica la dimensione in pixel del testo 'tariffa' e 'descrizione'
            pixel_x1, pixel_y1 = layout_descrizione.get_pixel_size()
            pixel_x2, pixel_y2 = layout_tariffa.get_pixel_size()
            #pixel_x2, pixel_y3 = layout_quantita.get_pixel_size()
            pixel_y = max(pixel_y1, pixel_y2)
            # aumenta la coordinata y in base al numero di pixel maggiore
            n_pixel_stampati += pixel_y
            ####################################################################
            # CONDIZIONI DI CAMBIO PAGINA ######################################
            ####################################################################
            # 1) cambio pagina se il tot. dei pixel = della dim. max pagina
            if n_pixel_stampati == self.n_pixel_pagina:
                layout_articolo = (layout_tariffa, layout_descrizione, layout_um, 
                                   layout_quantita, layout_prezzo_unitario, 
                                   layout_prezzo_totale, layout_costo_mat, 
                                   layout_ricarico, layout_tempo_inst,
                                   layout_sicurezza)
                lista_layout_articoli.append(layout_articolo)
                # salva il layout della pagina, aumenta il n. pagine e passa al prossimo articolo
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout_articoli
                operation.set_n_pages(self.num_pagine)
                self.articolo_stampare += 1
                return False
            # 2) cambio pagina se il tot. dei pixel > della dim. max pagina
            elif n_pixel_stampati > self.n_pixel_pagina:
                # calcola i pixel che rimangono da utilizzare nella pagina
                n_pixel_stampati -= pixel_y
                n_pixel_rimasti = self.n_pixel_pagina - n_pixel_stampati
                # cambio pagina se lo spazio rimasto di pagina < ai px. del testo tariffa
                if n_pixel_rimasti <= pixel_y2:
                    # salva il layout della pagina, aumenta il n. pagine, stampa l'articolo nella prossima pagina
                    self.num_pagine += 1
                    self.__layout_page[self.num_pagine] = lista_layout_articoli
                    operation.set_n_pages(self.num_pagine)
                    return False
                # crea una lista contenente le parole del testo
                list_words = text_desc.split(" ")
                new_text = str()
                # crea un layout vuoto per sostituire i campi da non stampare (x dopo)
                layout_vuoto = self.print_text_to_layout(context, '')
                # verifica ricorsivamente se la dimensione di un nuovo layout, creato diminuendo 
                # ciclicamente le parole del testo, finchè rientrano nello spazio di pagina rimanente
                while pixel_y1 > n_pixel_rimasti:
                    if len(list_words) == 0:
                        break
                    elif list_words[-1].endswith("\n"):
                        list_words[-1] = list_words[-1].rstrip("\n")
                    else:
                        word_erased = list_words.pop()
                    new_text = " ".join(list_words)
                    layout_descrizione = self.print_text_to_layout(context, new_text, width=width)
                    pixel_x1, pixel_y1 = layout_descrizione.get_pixel_size()
                # se il testo 'spezzato' non è vuoto crea l'articolo da stampare a fine pagina
                if new_text != str():
                    layout_articolo = (layout_tariffa, layout_descrizione,
                                       layout_vuoto, layout_vuoto, 
                                       layout_vuoto, layout_vuoto, 
                                       layout_vuoto, layout_vuoto,
                                       layout_vuoto, layout_vuoto)
                    lista_layout_articoli.append(layout_articolo)
                    # ricava la descrizione restante da stampare nella pagina successiva
                    testo_restante = text_desc.partition(new_text)[2]
                else:
                    testo_restante = text_desc
                # aggiungi il layout del nuovo articolo se il testo restante non è vuoto
                if testo_restante != str():
                    layout_descrizione = self.print_text_to_layout(context, testo_restante, width=width)
                    layout_articolo = (layout_vuoto, layout_descrizione, layout_um, 
                                   layout_quantita, layout_prezzo_unitario, 
                                   layout_prezzo_totale, layout_costo_mat, 
                                   layout_ricarico, layout_tempo_inst,
                                   layout_sicurezza)
                    self.__layout_articoli_riportati = layout_articolo
                # salva il layout della pagina, aumenta il n. pagine e passa al prossimo articolo
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout_articoli
                operation.set_n_pages(self.num_pagine)
                self.articolo_stampare += 1
                return False
            # 3) effettuare il cambio pagina alla fine di ogni categoria o capitolo
            elif articolo[0].startswith('$$$+$$$') and settings["new_chapter_change_page"]:
                # aggiungi i layout alla lista di articoli della pagina
                layout_articolo = (layout_tariffa, layout_descrizione, layout_um, 
                                   layout_quantita, layout_prezzo_unitario, 
                                   layout_prezzo_totale, layout_costo_mat, 
                                   layout_ricarico, layout_tempo_inst,
                                   layout_sicurezza)
                lista_layout_articoli.append(layout_articolo)
                # salva il layout della pagina, aumenta il n. pagine e passa al prossimo articolo
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout_articoli
                operation.set_n_pages(self.num_pagine)
                self.articolo_stampare += 1
                return False
            # 4) aggiungi i layout alla lista di articoli della pagina e prosegui il ciclo (no cambio pagina)
            else:
                layout_articolo = (layout_tariffa, layout_descrizione, layout_um, 
                                   layout_quantita, layout_prezzo_unitario, 
                                   layout_prezzo_totale, layout_costo_mat, 
                                   layout_ricarico, layout_tempo_inst,
                                   layout_sicurezza)
                lista_layout_articoli.append(layout_articolo)
                self.articolo_stampare += 1
        # prima di finire aggiungi l'ultima pagina
        if lista_layout_articoli != []:
            self.num_pagine += 1
            self.__layout_page[self.num_pagine] = lista_layout_articoli
            operation.set_n_pages(self.num_pagine)
        return True

################################################################################
########### STAMPA PER COMPUTO METRICO - EPU - LAVORAZIONI #####################
################################################################################

    def draw_computo_page(self, context, lista_layout_pagina, user_param=None):
        """Routine per disegnare le pagine da stampare in COMPUTO-EPU-LAVORAZIONI"""
        cairo_context = context.get_cairo_context()
        # disegno la griglia per il computo
        self.computo_grid(context)
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # spazio per intestazione
        y = HEADING_TOP
        # stampo tutti gli articoli della pagina
        for layout_articolo in lista_layout_pagina:
            #layout_articolo = (layout_tariffa, layout_descrizione, layout_um, layout_quantita, layout_prezzo_unitario, layout_prezzo_totale)
            # disegna una linea all'inizio dell'articolo
            if not layout_articolo[0].get_text() == '':
                self.draw_line(context, x0, y, BORDER_x, y, width = 0.5)
            # stampa codice e tariffa
            x = x0+ (0.010*BORDER_x) #10
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[0])
            # stampa descrizione (codice, estese e breve)
            x = x0+(0.11*BORDER_x) #80
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[1])
            # aumenta la variabile y
            pixel_x1, pixel_y1 = layout_articolo[0].get_pixel_size()
            pixel_x2, pixel_y2 = layout_articolo[1].get_pixel_size()
            y +=  max(pixel_y1, pixel_y2)-self.font_size
            # stampa um
            x = x0+(0.644*BORDER_x) #360
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[2])
            # stampa quantita
            x = x0+(0.716*BORDER_x) #400
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[3])
            # stampa prezzo unitario
            x = x0+(0.805*BORDER_x) #450
            if settings["computo_print_price"]:
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[4])
            # stampa prezzo totale
            x = x0+(0.894*BORDER_x) #500
            if settings["computo_print_price"]:
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[5])
            # aumenta il numero di pixel per compensare quanto precedentemente sottratto
            y += self.font_size
        return True

    def computo_grid(self, context, user_param=None):
        """crea la tabella per il COMPUTO-EPU-LAVORAZIONI"""
        cairo_context = context.get_cairo_context()
        # coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # linee orizzontali
        x1 = BORDER_x
        y1 = BORDER_y
        # linee per bordo pagina
        self.draw_line(context, x0, y0, x1, y0)
        self.draw_line(context, x0, y1, x1, y1)
        self.draw_line(context, x0, y1+15, x1, y1+15)
        # linee verticali
        self.draw_line(context, x0, y0, x0, y1+15)
        self.draw_line(context, x1, y0, x1, y1+15)
        # spazio per intestazione
        y2 = HEADING_TOP-30
        # linee per intestazione
        self.draw_line(context, x0, y2, x1, y2)
        self.draw_line(context, x0, HEADING_TOP, x1, HEADING_TOP)
        # intestazione delle colonne
        self.print_text(context, cairo_context, x0+(0.01*x1), y2, _("<b>Tariffa</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.11*x1), y2, _("<b>Descrizione</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.644*x1), y2, _("<b>U.M.</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.716*x1), y2, _("<b>Quantità</b>"), 
                        alignment = 0, width=(0.07*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.787*x1), y2, _("<b>Prezzo Unitario</b>"),
                        alignment = 0, width=(0.105*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.882*x1), y2, _("<b>Prezzo Totale</b>"),
                        alignment = 0, width=(0.105*x1*pango.SCALE))
        # linea per tariffa
        x = x0+(0.105*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per descrizione codice e descrizione breve
        x = x0+(0.635*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per um
        x = x0+(0.707*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per quantita
        x = x0+(0.796*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per prezzo unitario
        x = x0+(0.886*x1)
        self.draw_line(context, x, y2, x, y1)
        return True

################################################################################
########### STAMPA PER ANALISI #################################################
################################################################################
    def draw_analisi_page(self, context, lista_layout_pagina, user_param=None):
        """Routine per disegnare le pagine da stampare in COMPUTO con ricarichi e tempi"""
        cairo_context = context.get_cairo_context()
        # disegno la griglia per il computo
        self.analisi_grid(context)
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # spazio per intestazione
        y = HEADING_TOP 
        # stampo tutti gli articoli della pagina
        for layout_articolo in lista_layout_pagina:
            #layout_articolo = (cod_tariffa, Descrizione, um, quantita, pr_un, pr_tot, costo, ric, tempo, sic)
            #'$$$LIS-%s$$$' % articolo[1], Descrizione, um, quantita, pr_un, pr_tot, sconto, accessori, str()
            # disegna una linea all'inizio di ogni articolo
            if not layout_articolo[0].get_text().startswith("$$$LIS"):
                self.draw_line(context, x0, y, BORDER_x, y, width = 0.5)
                # stampa codice e tariffa
                x = x0+(0.009*BORDER_x)
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[0])
            # stampa descrizione codice e descrizione breve
            x = x0+(0.107*BORDER_x)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[1])
            # aumenta la variabile y
            pixel_x1, pixel_y1 = layout_articolo[0].get_pixel_size()
            pixel_x2, pixel_y2 = layout_articolo[1].get_pixel_size()
            y +=  max(pixel_y1, pixel_y2)-self.font_size
            # stampa um
            x = x0+(0.438*BORDER_x)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[2])
            # stampa quantita
            x = x0+(0.492*BORDER_x)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[3])
            # stampa prezzo unitario
            x = x0+(0.805*BORDER_x)
            if settings["computo_print_price"]:
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[4])
            # stampa prezzo totale
            x = x0+(0.876*BORDER_x)
            if settings["computo_print_price"]:
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[5])
            if not layout_articolo[0].get_text().startswith("$$$LIS"):
                # stampa costo materiali
                x = x0+(0.626*BORDER_x)
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[6])
                # stampa ricarichi
                x = x0+(0.698*BORDER_x)
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[7])
                # stampa tempo
                x = x0+(0.555*BORDER_x)
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[8])
                # stampa sicurezza
                x = x0+(0.751*BORDER_x)
                cairo_context.move_to(x, y)
                cairo_context.show_layout(layout_articolo[9])
            # aumenta il numero di pixel per compensare quanto precedentemente sottratto
            y += self.font_size   
        return True

    def analisi_grid(self, context, user_param=None):
        """crea la tabella per il COMPUTO-EPU-LAVORAZIONI"""
        cairo_context = context.get_cairo_context()
        # coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # linee orizzontali
        x1 = BORDER_x
        y1 = BORDER_y
        y2 = HEADING_TOP-30
        # linee per intestazione
        self.draw_line(context, x0, y2, x1, y2)
        self.draw_line(context, x0, HEADING_TOP, x1, HEADING_TOP)
        # linee bordo pagina
        self.draw_line(context, x0, y0, x1, y0)
        self.draw_line(context, x0, y1, x1, y1)
        self.draw_line(context, x0, y1+15, x1, y1+15)
        # linee verticali
        self.draw_line(context, x0, y0, x0, y1+15)
        self.draw_line(context, x1, y0, x1, y1+15)
        # intestazione delle colonne
        self.print_text(context, cairo_context, x0+(0.01*x1), y2, _("<b>Tariffa</b>"),
                        alignment = 0)
        self.print_text(context, cairo_context, x0+(0.11*x1), y2, _("<b>Descrizione</b>"),
                        alignment = 0)
        self.print_text(context, cairo_context, x0+(0.438*x1), y2, _("<b>U.M.</b>"),
                        alignment = 0, width=(0.052*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.492*x1), y2, _("<b>Quantità</b>"),
                        alignment = 0, width=(0.061*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.555*x1), y2, _("<b>Tempo (min)</b>"),
                        alignment = 0, width=(0.07*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.626*x1), y2, _("<b>Materiali</b>"),
                        alignment = 0, width=(0.07*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.698*x1), y2, _("<b>Ric.</b>"),
                        alignment = 0, width=(0.052*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.751*x1), y2, _("<b>Sic.(%)</b>"),
                        alignment = 0, width=(0.061*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.796*x1), y2, _("<b>Prezzo Unitario</b>"),
                        alignment = 0, width=(0.088*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.877*x1), y2, _("<b>Prezzo Totale</b>"),
                        alignment = 0, width=(0.088*x1*pango.SCALE))
        # linea per tariffa
        x = x0+(0.105*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per descrizione codice e descrizione breve
        x = x0+(0.635*x1)
        #self.draw_line(context, x, y2, x, y1)
        # linea per um
        x = x0+(0.438*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per quantita
        x = x0+(0.492*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per tempo
        x = x0+(0.555*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per materiali
        x = x0+(0.626*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per ricarico
        x = x0+(0.698*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per sicurezza
        x = x0+(0.751*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per prezzo unitario
        x = x0+(0.805*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per prezzo totale
        x = x0+(0.877*x1)
        self.draw_line(context, x, y2, x, y1)
        return True

################################################################################
########### IMPAGINAZIONE E STAMPA PER SCHEDA DI REPORT CONCLUSIVA #############
################################################################################
    def paginate_report(self, operation, context, user_param=None):
        """Procedura di impaginazione del report finale"""
        cairo_context = context.get_cairo_context()
        lista_layout = list()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # spazio per intestazione
        y = HEADING_TOP 
        # inizializzo il numero di righe stampate della pagina
        n_pixel_stampati = y
        # inizializzo la lista degli articoli da stampare
        lista = self.lista_report[self.rigo_report_stampare:]
        # stampa del report:
        for articolo in lista:
            # campi da stampare :'CAT', "numero cat", "nome", totale, incid_perc
            tipo = articolo[0]
            numero = articolo[1]
            nome = articolo[2]
            prezzo =  self.__convert_number_to_string(articolo[3])
            incidenza_perc = self.__convert_number_to_string(articolo[4])
            # stampa numero capitolo/categoria
            layout_numero = self.print_text_to_layout(context, numero, width=0.15*self.max_width)
            # stampa nome capitolo/categoria
            if tipo=='TYP':
                ind = None
                nome = "<big>%s</big>" % nome
            if tipo=='CAT':
                ind = 10000
                nome = "<i>%s</i>" % nome
            elif tipo=='SUB':
                ind = 20000
                nome = "<i>%s</i>" % nome
            elif tipo=='TOT':
                ind = None
                nome = "<b>%s</b>" % nome
                prezzo = "<b>%s</b>" % prezzo
            else: 
                ind=None
                nome = "<b>%s</b>" % nome
            layout_nome = self.print_text_to_layout(context, nome, width=1.1*self.max_width, indent=ind)
            # stampa prezzo
            layout_prezzo = self.print_text_to_layout(context, prezzo)
            # stampa incidenza percentuale
            layout_incidenza_perc = self.print_text_to_layout(context, incidenza_perc)
            # aumenta la coordinata y in base al numero di pixel della descrizione
            pixel_x1, pixel_y1 = layout_nome.get_pixel_size()
            pixel_x2, pixel_y2 = layout_numero.get_pixel_size()
            pixel_y = max(pixel_y1, pixel_y2)-self.font_size
            n_pixel_stampati += pixel_y+self.font_size
            # effettuare il cambio pagina quando lunghezza della descrizione supera la dim. max pagina
            if pixel_y >= self.n_pixel_pagina:
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                layout_articolo = (layout_numero, layout_nome, layout_prezzo, layout_incidenza_perc)
                lista_layout.append(layout_articolo)
                self.rigo_report_stampare += 1
                return False
            # effettuare il cambio il tot. stampato supera la dim. max pagina
            elif n_pixel_stampati > self.n_pixel_pagina:
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                return False
            elif tipo=='TOT':
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                layout_articolo = (layout_numero, layout_nome, layout_prezzo, layout_incidenza_perc)
                lista_layout.append(layout_articolo)
                self.rigo_report_stampare += 1
                return False
            # aggiungi i layout alla lista di articoli della pagina
            layout_articolo = (layout_numero, layout_nome, layout_prezzo, layout_incidenza_perc)
            lista_layout.append(layout_articolo)
            self.rigo_report_stampare += 1
        # prima di finire aggiungi l'ultima pagina
        if lista_layout != []:
            self.num_pagine += 1
            self.__layout_page[self.num_pagine] = lista_layout
            operation.set_n_pages(self.num_pagine)
        return True

    def draw_report_page(self, context, lista_layout_pagina, user_param=None):
        """Routine per disegnare le pagine da stampare"""
        cairo_context = context.get_cairo_context()
        # disegno la griglia per il computo
        self.report_grid(context)
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        y = HEADING_TOP # spazio per intestazione
        # stampo gli articoli della pagina:
        # layout_numero, layout_nome, layout_prezzo, layout_incidenza_perc
        for layout_articolo in lista_layout_pagina:
            y += self.font_size
            # stampa numero
            x = x0+(0.01*BORDER_x) #10
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[0])
            # stampa nome
            x = x0+(0.11*BORDER_x) #80
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[1])
            # stampa prezzo
            x = x0+(0.769*BORDER_x) #430
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[2])
            # stampa incidenza
            x = x0+(0.894*BORDER_x) #500
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[3])
            # aumenta la variabile y
            pixel_x1, pixel_y1 = layout_articolo[0].get_pixel_size()
            pixel_x2, pixel_y2 = layout_articolo[1].get_pixel_size()
            y +=  max(pixel_y1, pixel_y2)-self.font_size
        return True

    def report_grid(self, context, user_param=None):
        """crea la tabella per il computo metrico"""
        cairo_context = context.get_cairo_context()
        # coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # linee orizzontali
        x1 = BORDER_x
        y1 = BORDER_y
        # spazio per intestazione
        y2 = HEADING_TOP-30
        # linee per intestazione
        self.draw_line(context, x0, y2, x1, y2)
        self.draw_line(context, x0, HEADING_TOP, x1, HEADING_TOP)
        # linee per bordo
        self.draw_line(context, x0, y0, x1, y0)
        self.draw_line(context, x0, y1, x1, y1)
        self.draw_line(context, x0, y1+15, x1, y1+15)
        # linee verticali
        self.draw_line(context, x0, y0, x0, y1+15)
        self.draw_line(context, x1, y0, x1, y1+15)
        # intestazione delle colonne
        self.print_text(context, cairo_context, x0+(0.01*x1), y2, _("<b>Cod.</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.11*x1), y2, _("<b>Capitolo/Categoria</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.769*x1), y2, _("<b>Totale</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.894*x1), y2, _("<b>Inc.%</b>"), 
                        alignment = 0, width=(0.07*x1*pango.SCALE))
        # linea per nome capitolo/categoria
        x = x0+(0.105*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per prezzo
        x = x0+(0.760*x1) #425
        self.draw_line(context, x, y2, x, y1)
        # linea per incidenza percentuale
        x = x0+(0.886*x1) #495
        self.draw_line(context, x, y2, x, y1)
        return True

################################################################################
########### IMPAGINAZIONE E STAMPA PER INCIDENZA MANODOPERA ####################
################################################################################
    def paginate_report_incidenze(self, operation, context, user_param=None):
        """Procedura di impaginazione del report finale"""
        cairo_context = context.get_cairo_context()
        lista_layout = list()
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # spazio per intestazione
        y = HEADING_TOP
        # inizializzo il numero di righe stampate della pagina
        n_pixel_stampati = y
        # inizializzo la lista degli articoli da stampare
        lista = self.lista_report[self.rigo_report_stampare:]
        # stampa del report:
        for articolo in lista:
            # campi da stampare :'CAT', "numero cat", "nome", 'min_tot', 'totale', incid_perc
            tipo = articolo[0]
            numero = articolo[1]
            nome = articolo[2]
            tempo = self.__convert_number_to_string(articolo[3])
            prezzo =  self.__convert_number_to_string(articolo[4])
            incidenza_perc = self.__convert_number_to_string(articolo[5])
            # stampa numero capitolo/categoria
            layout_numero = self.print_text_to_layout(context, numero, width=0.15*self.max_width)
            # stampa nome capitolo/categoria  
            if tipo=='TYP':
                ind=None
                nome = "<big>%s</big>" % nome
            if tipo=='CAT':
                ind = 10000
                nome = "<i>%s</i>" % nome
            elif tipo=='SUB':
                ind = 20000
                nome = "<i>%s</i>" % nome
            elif tipo=='TOT':
                ind = None
                nome = "<b>%s</b>" % nome
                tempo = "<b>%s</b>" % tempo
                prezzo = "<b>%s</b>" % prezzo
            else: 
                ind=None
                nome = "<b>%s</b>" % nome
            layout_nome = self.print_text_to_layout(context, nome, width=1*self.max_width, indent=ind)
            # stampa tempo
            layout_tempo = self.print_text_to_layout(context, tempo)
            # stampa prezzo
            layout_prezzo = self.print_text_to_layout(context, prezzo)
            # stampa incidenza percentuale
            layout_incidenza_perc = self.print_text_to_layout(context, incidenza_perc)
            # aumenta la coordinata y in base al numero di pixel della descrizione
            pixel_x1, pixel_y1 = layout_nome.get_pixel_size()
            pixel_x2, pixel_y2 = layout_numero.get_pixel_size()
            pixel_y = max(pixel_y1, pixel_y2)-self.font_size
            n_pixel_stampati += pixel_y+self.font_size
            # effettuare il cambio pagina quando lunghezza della descrizione supera la dim. max pagina
            if pixel_y >= self.n_pixel_pagina:
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                layout_articolo = (layout_numero, layout_nome, layout_tempo, layout_prezzo, layout_incidenza_perc)
                lista_layout.append(layout_articolo)
                self.rigo_report_stampare += 1
                return False
            # effettuare il cambio il tot. stampato supera la dim. max pagina
            elif n_pixel_stampati > self.n_pixel_pagina:
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                return False
            elif tipo=='TOT' and True:
                # memorizza i layout
                self.num_pagine += 1
                self.__layout_page[self.num_pagine] = lista_layout
                operation.set_n_pages(self.num_pagine)
                layout_articolo = (layout_numero, layout_nome, layout_tempo, layout_prezzo, layout_incidenza_perc)
                lista_layout.append(layout_articolo)
                self.rigo_report_stampare += 1
                return False
            # aggiungi i layout alla lista di articoli della pagina
            layout_articolo = (layout_numero, layout_nome, layout_tempo, layout_prezzo, layout_incidenza_perc)
            lista_layout.append(layout_articolo)
            self.rigo_report_stampare += 1
        # prima di finire aggiungi l'ultima pagina
        if lista_layout != []:
            self.num_pagine += 1
            self.__layout_page[self.num_pagine] = lista_layout
            operation.set_n_pages(self.num_pagine)
        return True

    def draw_incidenze_page(self, context, lista_layout_pagina, report_type=None):
        """Routine per disegnare le pagine da stampare"""
        cairo_context = context.get_cairo_context()
        # disegno la griglia per il computo
        self.report_incidenze_grid(context, report_type)
        # impostazione delle coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        y = HEADING_TOP # spazio per intestazione
        # stampo gli articoli della pagina:
        # layout_numero, layout_nome, layout_prezzo, layout_incidenza_perc
        for layout_articolo in lista_layout_pagina:
            y += self.font_size
            # stampa numero
            x = x0+(0.01*BORDER_x)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[0])
            # stampa nome
            x = x0+(0.11*BORDER_x)
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[1])
            # stampa tempo
            x = x0+(0.644*BORDER_x) #360
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[2])
            # stampa prezzo
            x = x0+(0.769*BORDER_x) #430
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[3])
            # stampa incidenza
            x = x0+(0.894*BORDER_x) #500
            cairo_context.move_to(x, y)
            cairo_context.show_layout(layout_articolo[4])
            # aumenta la variabile y
            pixel_x1, pixel_y1 = layout_articolo[0].get_pixel_size()
            pixel_x2, pixel_y2 = layout_articolo[1].get_pixel_size()
            y +=  max(pixel_y1, pixel_y2)-self.font_size
        return True

    def report_incidenze_grid(self, context, report_type=None):
        """crea la tabella per il computo metrico"""
        # in base alla tipologia di report stabilisce il nome della colonna
        if report_type == "report-inc_mo":
             NOME_COLONNA = "Manodopera (ore)"
        elif report_type == "report-inc_sic":
             NOME_COLONNA = "Oneri di Sicurezza"
        else:
             NOME_COLONNA = "Oneri"
        cairo_context = context.get_cairo_context()
        # coordinate iniziali
        x0,y0 = self.__get_starting_coordinates()
        # linee orizzontali
        x1 = BORDER_x
        y1 = BORDER_y
        # spazio per intestazione
        y2 = HEADING_TOP-30
        # linee per intestazione
        self.draw_line(context, x0, y2, x1, y2)
        self.draw_line(context, x0, HEADING_TOP, x1, HEADING_TOP)
        #linee bordo
        self.draw_line(context, x0, y0, x1, y0)
        self.draw_line(context, x0, y1, x1, y1)
        self.draw_line(context, x0, y1+15, x1, y1+15)
        # linee verticali
        self.draw_line(context, x0, y0, x0, y1+15)
        self.draw_line(context, x1, y0, x1, y1+15)
        # intestazione delle colonne
        self.print_text(context, cairo_context, x0+(0.01*x1), y2, _("<b>Cod.</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.11*x1), y2, _("<b>Categoria</b>"), alignment = 0)
        self.print_text(context, cairo_context, x0+(0.644*x1), y2, _("<b>%s</b>" % NOME_COLONNA), 
                        alignment = 0, width=(0.114*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.769*x1), y2, _("<b>Totale Costo</b>"), 
                        alignment = 0, width=(0.105*x1*pango.SCALE))
        self.print_text(context, cairo_context, x0+(0.894*x1), y2, _("<b>Inc.%</b>"), 
                        alignment = 0, width=(0.07*x1*pango.SCALE))
        # linea per nome capitolo/categoria
        x = x0+(0.105*x1)
        self.draw_line(context, x, y2, x, y1)
        # linea per tempo
        x = x0+(0.635*x1) #355
        self.draw_line(context, x, y2, x, y1)
        # linea per prezzo
        x = x0+(0.760*x1) #425
        self.draw_line(context, x, y2, x, y1)
        # linea per incidenza percentuale
        x = x0+(0.886*x1) #495
        self.draw_line(context, x, y2, x, y1)
        return True

################################################################################
########### FUNZIONI DI DISEGNO CAIRO ##########################################
################################################################################

    def print_text(self, print_context, cairo_context, x, y, testo, font=None, 
                   width=None, indent=None, alignment=None):
        """"""
        if font is None:
           font = self.font
        if width is None:
           width = -1
        if indent is None:
           indent = 0
        if alignment is None:
           alignment = pango.ALIGN_LEFT
        elif alignment == 0:
           alignment = pango.ALIGN_CENTER
        elif alignment == 1:
           alignment = pango.ALIGN_RIGHT
        layout = print_context.create_pango_layout()
        layout.set_font_description(pango.FontDescription(font))
        layout.set_width(int(width))
        layout.set_wrap(pango.WRAP_WORD_CHAR)
        layout.set_alignment(alignment)
        layout.set_spacing(int(self.line_spacing))
        layout.set_indent(indent)
        layout.set_markup(testo)
        cairo_context.move_to(x, y)
        cairo_context.show_layout(layout)
        return layout.get_line_count()

    def print_text_to_layout(self, print_context, testo, font=None, width=None,
                             indent=None, alignment=None):
        """"""
        if font is None:
           font = self.font
        if width is None:
           width = -1
        if indent is None:
           indent = 0
        if alignment is None:
           alignment = pango.ALIGN_LEFT
        elif alignment == 0:
           alignment = pango.ALIGN_CENTER
        elif alignment == 1:
           alignment = pango.ALIGN_RIGHT
        layout = print_context.create_pango_layout()
        layout.set_font_description(pango.FontDescription(font))
        layout.set_width(int(width))
        layout.set_wrap(pango.WRAP_WORD_CHAR)
        layout.set_alignment(alignment)
        layout.set_spacing(int(self.line_spacing))
        layout.set_indent(indent)
        layout.set_markup(testo)
        # verificare se il testo è riconusciuto dal markup (spesso a causa del carattere '<')
        try:
            attrs, text, accel = pango.parse_markup(testo)
        except gobject.GError:
            logging.error("Errore di interpretazione del markup:\n%s" % testo)
        return layout

    def draw_line(self, context, x0, y0, x1, y1, width = None, rgb = None):
        """"""
        cairo_context = context.get_cairo_context()
        if width is None:
            width = self.width
        if rgb is None:
            r, g, b = self.color_rgb
        else: r, g, b = rgb
        cairo_context.set_line_width(width)
        cairo_context.set_source_rgb(r, g, b)
        cairo_context.set_line_cap(cairo.LINE_CAP_ROUND)
        cairo_context.set_line_join(cairo.LINE_JOIN_ROUND)
        cairo_context.move_to(x0, y0)
        cairo_context.line_to(x1, y1)
        cairo_context.stroke()
        # ritorna alle impostazioni iniziali
        cairo_context.set_source_rgb(0, 0, 0)
        return

    def draw_pixmap(self, context, png, x, y, max_height=None):
        """"""
        img = cairo.ImageSurface.create_from_png(png)
        height = img.get_height()
        if max_height is None:
            max_height = int(BORDER_y * MAX_HEADING)
        if  height > max_height:
            data = img.get_data()
            width = img.get_width()
            format = img.get_format() #cairo.FORMAT_ARGB32
            stride = cairo.ImageSurface.format_stride_for_width (format, width)
            new_height = min(height, max_height)
            img = cairo.ImageSurface.create_for_data(data, format, width, new_height, stride)
        cairo_context = context.get_cairo_context()
        cairo_context.save()
        cairo_context.set_source_surface(img, x, y)
        cairo_context.paint()
        cairo_context.restore()
        return img.get_height()

################################################################################
########### ALTRE FUNZIONI E CONVERSIONI #######################################
################################################################################
    def __format_description(self, descr_codice, descr_estesa=None, 
                             descr_voce=None, note=None, voce_analisi=False):
        """Formatta e compone la stringa della descrizione estesa"""
        text=str()
        # impostazioni campi ingresso nella funzione
        if descr_estesa is None: descr_estesa = str()
        if descr_voce is None: descr_voce = str()
        if note is None: note = str()
        # sostituzione dei caratteri '<' e '>' per evitare errori dell'istruzione 'layout.set_markup(testo)'
        descr_codice = descr_codice.replace("<", unichr(171)).replace(">", unichr(62))
        descr_estesa = descr_estesa.replace("<", unichr(171)).replace(">", unichr(62))
        descr_voce = descr_voce.replace("<", unichr(171)).replace(">", unichr(62))
        note = note.replace("<", unichr(171)).replace(">", unichr(62))
        # imposta i markup per i testi della descrizione
        codice_markup = settings["description_code_markup"]
        estesa_markup = settings["description_extended_markup"]
        voce_markup = settings["description_voice_markup"]
        note_markup = settings["description_note_markup"]
        analisi_markup = settings["description_analysis_markup"]
        # crea il testo descrizione dell'articolo con markup
        if codice_markup != str() and not voce_analisi:
            txt_codice = []
            for word in descr_codice.split(" "):
                word = '<{markup}>{0}</{markup}>'.format(word, markup=codice_markup)
                txt_codice.append(word)
            text += " ".join(txt_codice) + "\n"
        # crea gli articoli di analisi con con markup
        elif analisi_markup != str() and voce_analisi:
            txt_analisi = []
            for word in descr_codice.split(" "):
                word = '<{markup}>{0}</{markup}>'.format(word, markup=analisi_markup)
                txt_analisi.append(word)
            text += " ".join(txt_analisi) + "\n"
        else:
            text += "%s\n" % descr_codice
        if settings["computo_print_descr_estesa"] and descr_estesa != str():
            if estesa_markup != str():
                txt_estesa = []
                for word in descr_estesa.split(" "):
                    word = '<{markup}>{0}</{markup}>'.format(word, markup=estesa_markup)
                    txt_estesa.append(word)
                text += " ".join(txt_estesa) + "\n"
            else:
                text += "%s\n" % descr_estesa
        if settings["computo_print_descr_voce"] and descr_voce != str():
            if voce_markup != str():
                txt_voce = []
                for word in descr_voce.split(" "):
                    word = '<{markup}>{0}</{markup}>'.format(word, markup=voce_markup)
                    txt_voce.append(word)
                text += " ".join(txt_voce) + "\n"
            else:
                text += "%s\n" % descr_voce
        if settings["computo_print_note"] and note != str():
            if note_markup != str():
                txt_note = []
                for word in note.split(" "):
                    word = '<{markup}>{0}</{markup}>'.format(word, markup=note_markup)
                    txt_note.append(word)
                text += " ".join(txt_note) + "\n"
            else:
                text += "%s\n" % note
        if text.endswith("\n"): text = text.rstrip("\n")
        return text

    def __convert_number_to_string(self, number, round_to_integer = False, decimals = None):
        if decimals is None:
            decimals = int(settings["decimals"])
        # se i decimali del numero sono 0 trasformalo in un intero
        if type(number) is float and round_to_integer:
            if number.is_integer():
                number = int(number)
        # se il numero è con virgola usa 2 decimali - per le analisi occorre usare 3 cifre decimali!!!
        if type(number) is float:
            number = round(number, decimals)
            text = locale.format("%.{0}f".format(decimals), number, grouping=True)
        elif type(number) is int:
            text = locale.format("%d", number, grouping=True)
        # se l'input è una stringa ricava i valori numerici 
        elif type(number) is str and number != '':
            text = number
            # prova a trasformare il testo in numero per poter verificare poi se è negativo
            try:
                number = locale.atof(number)
            except ValueError:
                pass
        else:
            text = locale.format("%s", number, grouping=True)
        # se il numero è negativo viene stampato in rosso
        if number < 0:
            text = """<span foreground="red">{0}</span>""".format(text)
        return text

    def __get_current_date(self):
        data = str(datetime.datetime.now())[:10]
        data = "%s-%s-%s" % (data[8:10], data[5:7], data[:4])
        return data

    def __get_starting_coordinates(self):
        x = int(settings["page_offset_top_x"])
        y = int(settings["page_offset_top_y"])
        return (min(MAX_OFFSET,x), min(MAX_OFFSET,y))

    def __get_bottom_offset(self):
        x = int(settings["page_offset_bottom_x"])
        y = int(settings["page_offset_bottom_y"])
        return (min(MAX_OFFSET,x), min(MAX_OFFSET,y))

    def __get_top_heading(self, context=None):
        x0,y0 = self.__get_starting_coordinates()
        std_head = 15 # spazio fisso per il titolo del computo
        column_head = 30 # spazio fisso per intestazione delle colonne del computo
        if settings["print_logo"] and context is not None:
            height = self.draw_pixmap(context, self.owner_image, x0, y0)
        else: 
            height = 0
        return y0+height+column_head+std_head

    def __get_bottom_heading(self):
        x,y = self.__get_bottom_offset()
        return y-self.font_size


################################################################################
########### ESPORTAZIONE SU FILE CSV ###########################################
################################################################################
class CsvPreventa:
    def __init__(self, preventivo, filename, tipo_stampa, csv_settings=None,
                 categorie=None, capitoli=None):
        self.preventivo = preventivo
        # valori accettati della CLASSE: "computo","epu", "lavorazioni", "analisi"
        self.tipo_stampa = tipo_stampa
        # impostazione dei settaggi di default
        if csv_settings is None:
            settings = DEFAULT_SETTINGS
        elif type(csv_settings) is type(gtk.PrintSettings()):
            # decodifica le impostazioni (gtk.PrintSettings)
            settings = dict()
            csv_settings.foreach(self.__setup_key, user_data=settings)
        else:
            # messaggio di errore
            logging.error(_("Errore impostazioni di esportazione in CSV errate"))
        # ricava le informazioni da esportare
        self.lista_articoli, self.lista_report = get_output_to_print(
                                    self.preventivo, settings, self.tipo_stampa,
                                    categorie=categorie, capitoli=capitoli)
        # migliora l'output
        self.lista_articoli = self.__setup_articoli(self.lista_articoli) 
        self.lista_report = self.__setup_report(self.lista_report)
        # are il file csv in scrittura
        file_csv = csv.writer(open(filename, 'wb'), dialect=csv.excel, 
                                 quoting=csv.QUOTE_NONNUMERIC)
        # scrive sul file i dati estratti e processati
        file_csv.writerow([_("No.\nTariffa"), _("Descrizione"), _("UM"), 
                              _("Quantita"), _("P.Unitario"), _("P.Totale")])
        file_csv.writerows(self.lista_articoli)
        # scrive sul file il report finale
        file_csv.writerow([_("Cat."), _("Categoria\n"), "", _("Totale"),
                                  _("Incidenza"), ""])
        file_csv.writerows(self.lista_report)
        logging.debug(_("Esportazione formato CSV tipo stampa {0}, file: {1}").format(
                        tipo_stampa, filename))

    def __setup_articoli(self, lista):
        lista_layout = list()
        for art in lista:
            code, tariffa, desc, um, quant, pu, pt, costo, ric, tempo, sic = art
            # modifiche al campo descrizione
            codice, estesa, breve, note = desc
            if estesa != str(): estesa = "\n"+estesa
            if breve != str(): breve = "\n"+breve
            if note != str(): note = "\n"+note
            descrizione = "{0}{1}{2}{3}".format(codice, estesa, breve, note)
            # aggiungi alla lista il nuovo articolo (esclude i campi: costo, ric, tempo, sic)
            lista_layout.append((tariffa, descrizione, um, quant, pu, pt))
        return lista_layout

    def __setup_report(self, lista):
        lista_layout = list()
        vuota = str()
        for art in lista:
            #('SUP', '1', 'IMPIANTO DI ILLUMINAZIONE\n', 214339.89252000011, 26.822546587931882)
            cod, tariffa, descrizione, pt, inc = art
            # modifiche al campo incidenza
            if type(inc) is float or type(inc) is int:
                inc = "{0:.2%}".format(inc/100)
            # aggiungi alla lista il nuovo articolo
            lista_layout.append((tariffa, descrizione, vuota, pt, inc, vuota))
        return lista_layout

    def __setup_key(self, key, value, settings):
        if value == 'false':
            value = False
        elif value == 'true':
            value = True
        settings[key] = value
        return key, value
################################################################################
########### PER RICAVARE DAL PREVENTIVO LA LISTA ARTICOLI DA STAMPARE !!! ######
################################################################################
def get_output_to_print(preventivo, settings, tipo_stampa, categorie=None, capitoli=None):
    """Ottiene dal preventivo la lista degli articoli da stampare a seconda del tipo stampa"""
    # ricava la lista dei capitoli e delle categorie da stampare
    if categorie is not None:
        sup_cat, cat, sub_cat = categorie
    else: sup_cat, cat, sub_cat = (None, None, None)
    if capitoli is not None:
        sup_cap, cap, sub_cap = capitoli
    else: sup_cap, cap, sub_cap = (None, None, None)
    # ricavare la stampa totale del computo
    lista_articoli_stampare = list()
    lista_report = list()
    if tipo_stampa == "computo":
        lista_articoli_stampare = preventivo.print_computo (
                       lista_supercategorie=sup_cat, lista_categorie=cat,
                       lista_subcategorie=sub_cat, 
                       stampa_descr_breve = settings["computo_print_descr_voce"], 
                       stampa_descr_estesa = settings["computo_print_descr_estesa"], 
                       stampa_note = settings["computo_print_note"],
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"],
                       print_without_chapter = settings["print_without_chapter"],
                       print_analisi= settings["print_analisi"])
        # report finale per computo
        if settings["print_report_by_chapter"]:
            lista_report.extend(preventivo.print_report_categorie(
                       lista_supercategorie=sup_cat, lista_categorie=cat,
                       lista_subcategorie=sub_cat,
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
            lista_report.extend(preventivo.print_report_computo (lista_supercategorie=sup_cat, 
                       lista_categorie=cat, lista_subcategorie=sub_cat, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"], 
                       print_only_grand_total = settings["print_only_grand_total"]))
    elif tipo_stampa == "epu":
        lista_articoli_stampare = preventivo.print_epu (
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap,
                       stampa_descr_breve = settings["computo_print_descr_voce"], 
                       stampa_descr_estesa = settings["computo_print_descr_estesa"], 
                       stampa_note = settings["computo_print_note"],
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"],
                       print_without_chapter = settings["print_without_chapter"],
                       print_analisi= settings["print_analisi"])
    elif tipo_stampa == "lavorazioni":
        lista_articoli_stampare = preventivo.print_lavorazioni( 
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap, 
                       stampa_descr_breve = settings["computo_print_descr_voce"], 
                       stampa_descr_estesa = settings["computo_print_descr_estesa"], 
                       stampa_note = settings["computo_print_note"],
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"],
                       print_without_chapter = settings["print_without_chapter"],
                       print_analisi= settings["print_analisi"])
        # report finale per lista delle lavorazioni
        if settings["print_report_by_chapter"]:
            lista_report.extend(preventivo.print_report_capitoli (
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
            lista_report.extend(preventivo.print_report_lavorazioni(
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"], 
                       print_only_grand_total = settings["print_only_grand_total"]))
    elif tipo_stampa == "analisi":
        lista_articoli_stampare = preventivo.print_computo (lista_supercategorie=sup_cat, 
                       lista_categorie=cat, lista_subcategorie=sub_cat, 
                       stampa_descr_breve = settings["computo_print_descr_voce"], 
                       stampa_descr_estesa = settings["computo_print_descr_estesa"], 
                       stampa_note = settings["computo_print_note"],
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"],
                       print_without_chapter = settings["print_without_chapter"],
                       print_analisi= True)
    elif tipo_stampa == "report-cmp":
        lista_report = preventivo.print_report_categorie (lista_supercategorie=sup_cat, 
                       lista_categorie=cat, lista_subcategorie=sub_cat,
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"])
        lista_report.extend(preventivo.print_report_computo (lista_supercategorie=sup_cat, 
                       lista_categorie=cat, lista_subcategorie=sub_cat, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"], 
                       print_only_grand_total = False))
    elif tipo_stampa == "report-epu":
        lista_articoli_stampare = list()
        lista_report.extend(preventivo.print_report_capitoli (
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
        lista_report.extend(preventivo.print_report_lavorazioni(
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, lista_subcapitoli=sub_cap, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"], 
                       print_only_grand_total = settings["print_only_grand_total"]))
    elif tipo_stampa == "report-inc_mo":
        lista_report.extend(preventivo.print_report_inc_manodopera (
                       lista_supercategorie=sup_cat, lista_categorie=cat,
                       lista_subcategorie=sub_cat,
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
        # stampare report incidenza della manodopera su lista dei capitoli di epu
        lista_report.extend(preventivo.print_report_inc_mo_lavorazioni ( 
                       lista_supercapitoli=sup_cap, lista_capitoli=cap, 
                       lista_subcapitoli=sub_cap, 
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
    elif tipo_stampa == "report-inc_sic":
        lista_report.extend(preventivo.print_report_inc_sicurezza (
                       lista_supercategorie=sup_cat, lista_categorie=cat,
                       lista_subcategorie=sub_cat,
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
        # stampare report incidenza della manodopera su lista dei capitoli di epu
        lista_report.extend(preventivo.print_report_inc_sic_lavorazioni (
                       lista_supercapitoli=sup_cap, lista_capitoli=cap,
                       lista_subcapitoli=sub_cap,
                       print_only_if_not_0 = settings["print_only_if_quantity_not_0"]))
        #lista_report.extend(preventivo.print_report_inc_sic_lav (...)
    return (lista_articoli_stampare, lista_report)
