# -*- Mode: Python; coding: utf-8; indent-tabs-mode: nil; tab-width: 4 -*-
### BEGIN LICENSE
# Copyright (C) 2011 <Davide Vescovini> <davide.vescovini@gmail.com>
# This program is free software: you can redistribute it and/or modify it 
# under the terms of the GNU General Public License version 3, as published 
# by the Free Software Foundation.
# 
# This program is distributed in the hope that it will be useful, but 
# WITHOUT ANY WARRANTY; without even the implied warranties of 
# MERCHANTABILITY, SATISFACTORY QUALITY, or FITNESS FOR A PARTICULAR 
# PURPOSE.  See the GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License along 
# with this program.  If not, see <http://www.gnu.org/licenses/>.
### END LICENSE

# THIS IS Preventares CONFIGURATION FILE
# YOU CAN PUT THERE SOME GLOBAL VALUE
# Do not touch unless you know what you're doing.
# you're warned :)

######################################################
########### VARIABILI BASE  ##########################
######################################################

__all__ = [
    'project_path_not_found',
    'get_data_file',
    'get_data_path',
    ]

# Where your project will look for your data (for instance, images and ui
# files). By default, this is ../data, relative your trunk layout
__preventares_data_directory__ = '../data/'
__license__ = 'GPL-3'

# Impostazioni file configurazione e archivi principali e metel
__user_directory__ = '~'
__preventares_config_directory__ = '~/Preventa/'
__preventares_archive_directory__ = 'archive'
__preventares_archive_filename__ = 'archive.db'
__preventares_metel_filename__ = 'metel.db'
__preventares_config_file__ = '.preventares.cfg'
__preventares_print_settings_file__ = '.print_settings.cfg'
######################################################
########### IMPORTAZIONE LIBRERIE  ###################
######################################################
import os
import gettext
from gettext import gettext as _
gettext.textdomain('preventares')

######################################################
########### GESTIONE ERRORI  #########################
######################################################

class project_path_not_found(Exception):
    """Raised when we can't find the project directory."""

######################################################
########### DIRECTORY 'DATA' #########################
######################################################

def get_data_file(*path_segments):
    """Get the full path to a data file.

    Returns the path to a file underneath the data directory (as defined by
    `get_data_path`). Equivalent to os.path.join(get_data_path(),
    *path_segments).
    """
    return os.path.join(get_data_path(), *path_segments)


def get_data_path():
    """Retrieve preventares data path

    This path is by default <preventares_lib_path>/../data/ in trunk
    and /usr/share/preventares in an installed version but this path
    is specified at installation time.
    """

    # Get pathname absolute or relative.
    path = os.path.join(
        os.path.dirname(__file__), __preventares_data_directory__)

    abs_data_path = os.path.abspath(path)
    if not os.path.exists(abs_data_path):
        raise project_path_not_found

    return abs_data_path

######################################################
########### FILE CONFIGURAZIONE  #####################
######################################################
def get_config_path():
    """Retrieve preventares user configuration path: $HOME/$CONFIGURATION_PATH/"""
    # Get pathname absolute or relative.
    home = os.path.expanduser(__preventares_config_directory__)
    path = os.path.join(
        os.path.dirname(__file__), home)
    abs_data_path = os.path.abspath(path)
    return abs_data_path
def get_config_file():
    """Get the full path to the configuration file."""
    return os.path.join(get_config_path(), __preventares_config_file__)
def get_print_settings_file():
    """Get the full path to the printing settings file."""
    return os.path.join(get_config_path(), __preventares_print_settings_file__)
def get_archive_path(*path_segments):
    """Get the full path to an archive data file."""
    return os.path.join(get_config_path(), __preventares_archive_directory__, *path_segments)
def get_archive_filename():
    """Get the full path to the archive db filename."""
    return os.path.join(get_archive_path(), __preventares_archive_filename__)
def get_metel_filename():
    """Get the full path to the metel db filename."""
    return os.path.join(get_archive_path(), __preventares_metel_filename__)
def get_user_path():
    """Retrieve preventares's user main path (es: $HOME)"""
    # Get pathname absolute or relative.
    path = os.path.expanduser(__user_directory__)
    abs_data_path = os.path.abspath(path)
    return abs_data_path
